@@ -10,7 +10,7 @@
 
 if (FALSE){
   library(ff)
-  n <- 10000
+  n <- 160
   x <- ff(as.factor(letters[1:16]), dim=c(n,4), dimorder=2:1)
   y <- ff(1:(n*4), dim=c(n,4), dimorder=2:1)
   colnames(y) <- letters[1:4]
@@ -25,9 +25,9 @@ if (FALSE){
   ffvecapply(rnam[i1:i2] <- as.character.hexmode(i1:i2), N=n, VMODE="double", VERBOSE=TRUE)
 
   #d <- ffdf(x, y, z, z2, z3)
-  #dj <- ffdf(x, y, z, z2, z3, join=list(c(1,3), c(2,4,5)), ff_args=list(pattern="dj_"))
+  #dj <- ffdf(x, y, z, z2, z3, ff_join=list(c(1,3), c(2,4,5)), ff_args=list(pattern="dj_"))
   #di <- ffdf(I(x), y, z, z2, z3)
-  #dji <- ffdf(I(x), y, z, z2, z3, join=list(c(1,3), c(2,4,5)), ff_args=list(pattern="dj_"))
+  #dji <- ffdf(I(x), y, z, z2, z3, ff_join=list(c(1,3), c(2,4,5)), ff_args=list(pattern="dj_"))
 
   d   <- ffdf(x, y, z, z2, z3, row.names=rnam)
   dj  <- ffdf(x, y, z, z2, z3, ff_join=list(c(1,3), c(2,4,5)), row.names=rnam, ff_args=list(pattern="dj_"))
@@ -734,9 +734,9 @@ ffdf <- function(
           if (n==1 && is.null(dim(dotval[[sj[1]]]))){
             # store physically as ff_vector
             f <- dotval[[sj[1]]]
-            fnam <- paste(sjinam, "vec", sep=".")
+            fnam <- paste(sjinam, "vec.", sep=".")
             useffargs <- ff_args
-            useffargs[c("vmode", "length", "filename")] <- list(sjmodes, nrows, paste(ff_args$pattern, fnam, getOption("ffextension"), sep="."))
+            useffargs[c("vmode", "length", "pattern")] <- list(sjmodes, nrows, paste(ff_args$pattern, fnam, sep="."))
             physical[[p]] <- do.call("clone", c(list(f), useffargs))
             names(physical)[p] <- fnam
             if (inherits(f, "AsIs")){
@@ -762,9 +762,9 @@ ffdf <- function(
                 anyAsIs <- TRUE
               }
               if (i==1){
-                fnam <- paste(sjinam, "mat", sep=".")
+                fnam <- paste(sjinam, "mat.", sep=".")
                 useffargs <- ff_args
-                useffargs[c("vmode", "dim","dimorder","filename")] <- list(names(maxffmode(sjmodes)), c(nrows, n), 2:1, paste(ff_args$pattern, fnam, getOption("ffextension"), sep="."))
+                useffargs[c("vmode", "dim","dimorder","pattern")] <- list(names(maxffmode(sjmodes)), c(nrows, n), 2:1, paste(ff_args$pattern, fnam, sep="."))
 
                 physical[[p]] <- do.call("clone", c(list(f), useffargs))
                 colnames(physical[[p]]) <- pnam[offset+seq.int(length.out=n)]
@@ -1157,7 +1157,7 @@ ffdf <- function(
         }else{
           names(df) <- row.names(virtual)
           if (nrows!=1 || !drop){
-            df <- as.data.frame(df)
+            class(df) <- "data.frame"
           }
         }
       }else{
@@ -1169,28 +1169,35 @@ ffdf <- function(
           i2 <- as.hi(i, maxindex=nvw$n, vw=nvw$vw, pack=FALSE, envir=parent.frame(), names=rownam)
           nrows <- length(i2)
         }
-        df <- data.frame(row.names=seq.int(length.out=nrows))
+        df <- list()
+        class(df) <- "data.frame"
       }
 
-
-      if (is.data.frame(df) && !is.null(rownam)){
-        if (is.ff(rownam)){
-          nvw <- get_nvw(rownam)
-          if (!identical(last_nvw, nvw)){
-            if (missing(i))
-              i2 <- hi(from=1, to=nvw$n, maxindex=nvw$n, vw=nvw$vw, pack=FALSE)
-            else{
-              i2 <- as.hi(i, maxindex=nvw$n, vw=nvw$vw, pack=FALSE, envir=parent.frame(), names=rownam)
-            }
-          }
-          row.names(df) <- rownam[i2]
+      if (is.data.frame(df)){
+        if (is.null(rownam)){
+          if (missing(i))
+            row.names(df) <- seq.int(length.out=nrows)
+          else
+            row.names(df) <- as.which(i2)
         }else{
-          if (missing(i)){
-            row.names(df) <- rownam
-          }else if(is.character(i))
-            row.names(df) <- i
-          else{
-            row.names(df) <- rownam[as.integer(as.hi(i, maxindex=nvw$n, vw=nvw$vw, pack=FALSE, envir=parent.frame(), names=rownam))]
+          if (is.ff(rownam)){
+            nvw <- get_nvw(rownam)
+            if (!identical(last_nvw, nvw)){
+              if (missing(i))
+                i2 <- hi(from=1, to=nvw$n, maxindex=nvw$n, vw=nvw$vw, pack=FALSE)
+              else{
+                i2 <- as.hi(i, maxindex=nvw$n, vw=nvw$vw, pack=FALSE, envir=parent.frame(), names=rownam)
+              }
+            }
+            row.names(df) <- rownam[i2]
+          }else{
+            if (missing(i)){
+              row.names(df) <- rownam
+            }else if(is.character(i))
+              row.names(df) <- i
+            else{
+              row.names(df) <- rownam[as.integer(as.hi(i2))]
+            }
           }
         }
       }
@@ -1475,14 +1482,15 @@ is.ffdf <- function(x)
 #! as.ffdf(x, ...)
 #! \method{as.ffdf}{ff_vector}(x, ...)
 #! \method{as.ffdf}{ff_matrix}(x, ...)
-#! \method{as.ffdf}{data.frame}(x, vmode=NULL, ...)
+#! \method{as.ffdf}{data.frame}(x, vmode=NULL, col_args = list(), ...)
 #! \method{as.data.frame}{ffdf}(x, ...)
 #! }
 #! \arguments{
 #!   \item{x}{ the object to be coerced }
 #!   \item{vmode}{ optional specification of the \code{\link{vmode}s} of columns of the \code{\link{data.frame}}. Either a character vector of vmodes (named with column names of the data.frame or recycled if not named)
 #!                 or a list named with vmodes where each element identifies those columns of the data.frame that should get the vmode encoded in the name of the element }
-#!   \item{\dots}{ further arguments; passed to \code{\link{ffdf}} for .ff_vector and .ff_matrix methods, passed to \code{\link{ff}} for .data.frame method, ignored for .ffdf identity method }
+#!   \item{col_args}{ further arguments; passed to \code{\link{ff}}  }
+#!   \item{\dots}{ further arguments; passed to \code{\link{ffdf}} for .ff_vector, .ff_matrix and .data.frame methods, ignored for .ffdf identity method }
 #! }
 #! \value{
 #!   'as.ffdf' returns an object of class \code{\link{ffdf}}, 'as.data.frame' returns an object of class \code{\link{data.frame}}
@@ -1508,7 +1516,12 @@ as.ffdf.ff_matrix <- function(x, ...){
 as.ffdf.ff_vector <- function(x, ...){
   ffdf(x, row.names=names(x), ...)
 }
-as.ffdf.data.frame <- function(x, vmode=NULL, ...){
+as.ffdf.data.frame <- function(
+  x
+, vmode = NULL
+, col_args=list()
+, ...
+){
   rnam <- attr(x, "row.names")
   if (is.integer(rnam)){
     if (all(rnam==seq.int(along=rnam)))
@@ -1540,26 +1553,29 @@ as.ffdf.data.frame <- function(x, vmode=NULL, ...){
       }
     }
   }
+  if (is.null(col_args$pattern))
+    col_args$pattern <- "ffdf"
+
   l <- list(...)
-  if (is.null(l$pattern))
-    l$pattern <- "ffdf"
+  if (is.null(l$ff_args$pattern))
+    l$ff_args$pattern <- "ffdf"
   ret <- lapply(seq.int(along=x)
   , function(i, ...){
     xi <- x[[i]]
     AsIs <- inherits(xi, "AsIs")
     if (AsIs){
       oldClass(xi) <- oldClass(xi)[-match("AsIs", oldClass(xi))]
-      ret <- as.ff(xi, vmode=vmodes[[i]], ...)
+      ret <- do.call("as.ff", c(list(xi, vmode=vmodes[[i]]), col_args))
       oldClass(ret) <- c("AsIs", oldClass(ret))
       ret
     }else{
-      do.call("as.ff", c(list(xi, vmode=vmodes[[i]]), l))
+      do.call("as.ff", c(list(xi, vmode=vmodes[[i]]), col_args))
     }
   }
   , ...
   )
   names(ret) <- names(x)
-  do.call("ffdf", c(ret, list(row.names=rnam)))
+  do.call("ffdf", c(ret, list(row.names=rnam), l))
 }
 
 as.data.frame.ffdf <- function(x, ...)
@@ -1648,22 +1664,28 @@ vmode.ffdf <- function(x, ...){
 
 
 
+
+
+
 #! \name{chunk.ffdf}
 #! \Rdversion{1.1}
 #! \alias{chunk.ffdf}
+#! \alias{chunk.ff_vector}
 #! \title{
-#!    Chunk ffdf
+#!    Chunk ff_vector and ffdf
 #! }
 #! \description{
-#!    Row-wise chunking method for ffdf objects automatically considering RAM requirements from recordsize as calculated from \code{\link{sum}(\link{.rambytes}[\link[=vmode.ffdf]{vmode}])}
+#!    Chunking method for ff_vector and ffdf objects (row-wise) automatically considering RAM requirements from recordsize as calculated from \code{\link{sum}(\link{.rambytes}[\link{vmode}])}
 #! }
 #! \usage{
-#! \method{chunk}{ffdf}(x, \dots, BATCHBYTES = getOption("ffbatchbytes"))
+#! \method{chunk}{ff_vector}(x, RECORDBYTES = .rambytes[vmode(x)], BATCHBYTES = getOption("ffbatchbytes"), \dots)
+#! \method{chunk}{ffdf}(x, RECORDBYTES = sum(.rambytes[vmode(x)]), BATCHBYTES = getOption("ffbatchbytes"), \dots)
 #! }
 #! \arguments{
-#!   \item{x}{\code{\link{ffdf}}}
-#!   \item{\dots}{further arguments passed to \code{\link[bit]{chunk}}}
+#!   \item{x}{\code{\link{ff}} or \code{\link{ffdf}}}
+#!   \item{RECORDBYTES}{ optional integer scalar representing the bytes needed to process an element of the \code{ff_vector} a single row of the \code{ffdf} }
 #!   \item{BATCHBYTES}{ integer scalar limiting the number of bytes to be processed in one chunk, default from \code{getOption("ffbatchbytes")}, see also \code{\link{.rambytes}} }
+#!   \item{\dots}{further arguments passed to \code{\link[bit]{chunk}}}
 #! }
 #! \value{
 #!   A list with \code{\link[bit]{ri}} indexes each representing one chunk
@@ -1708,7 +1730,33 @@ vmode.ffdf <- function(x, ...){
 #! \keyword{ data }
 
 
-chunk.ffdf <- function(x, ..., BATCHBYTES = getOption("ffbatchbytes")){
+chunk.ff_vector <- function(x, RECORDBYTES = .rambytes[vmode(x)], BATCHBYTES = getOption("ffbatchbytes"), ...){
+  n <- length(x)
+  if (n){
+    l <- list(...)
+    if (is.null(l$from))
+      l$from <- 1L
+    if (is.null(l$to))
+      l$to <- n
+    if (is.null(l$by) && is.null(l$len)){
+      b <- BATCHBYTES %/% RECORDBYTES
+      if (b==0L){
+        b <- 1L
+        warning("single record does not fit into BATCHBYTES")
+      }
+      l$by <- b
+    }
+    l$maxindex <- n
+    ret <- do.call("chunk.default", l)
+
+  }else{
+    ret <- list()
+  }
+  ret
+}
+
+
+chunk.ffdf <- function(x, RECORDBYTES = sum(.rambytes[vmode(x)]), BATCHBYTES = getOption("ffbatchbytes"), ...){
   n <- nrow(x)
   if (n){
     l <- list(...)
@@ -1717,14 +1765,14 @@ chunk.ffdf <- function(x, ..., BATCHBYTES = getOption("ffbatchbytes")){
     if (is.null(l$to))
       l$to <- n
     if (is.null(l$by) && is.null(l$len)){
-      recordsize <- sum(.rambytes[vmode(x)])
-      b <- BATCHBYTES %/% recordsize
+      b <- BATCHBYTES %/% RECORDBYTES
       if (b==0L){
         b <- 1L
         warning("single record does not fit into BATCHBYTES")
       }
       l$by <- b
     }
+    l$maxindex <- n
     ret <- do.call("chunk.default", l)
 
   }else{
