@@ -1,7 +1,12 @@
 //#ifndef FF_XTENSIONS_AVAILABLE
-#define FF_XTENSIONS_AVAILABLE 0
+#define FF_XTENSIONS_AVAILABLE 1
 //#endif
 
+//#ifndef FF_SYMMXTENSIONS_AVAILABLE
+#define FF_SYMMXTENSIONS_AVAILABLE 0
+//#endif
+
+
 #ifndef FF_CONFIG_H
 #define FF_CONFIG_H
 
