@@ -4,15 +4,18 @@
 # Licence: GPL2
 # Provided 'as is', use at your own risk
 # Created: 2007-08-24
-# Last changed: 2007-11-29
+# Last changed: 2009-12-12
 */
 
 /* All combinations of
    - getset, get set addgetset addset
-   - (single), contiguous, vector, array
+   - (single), contiguous, vec, vector, array
    - boolean, logical, quad, nibble, byte, ubyte, short, ushort, integer, single, double, complex
 */
 
+
+/* { -- getset ----------------------------------------------------------
+
 SEXP r_ff__getset(      SEXP ffmode_, SEXP ff_, SEXP index_, SEXP value_);
 SEXP r_ff__get(         SEXP ffmode_, SEXP ff_, SEXP index_             );
 SEXP r_ff__set(         SEXP ffmode_, SEXP ff_, SEXP index_, SEXP value_);
@@ -111,8 +114,11 @@ SEXP r_ff_raw_addgetset(   SEXP ff_, SEXP index_, SEXP value_);
 SEXP r_ff_raw_addset(      SEXP ff_, SEXP index_, SEXP value_);
 #endif
 
+ } -- getset ---------------------------------------------------------- */
 
 
+/* { -- getset_contiguous ---------------------------------------------------------- */
+
 SEXP r_ff__getset_contiguous(   SEXP ffmode_, SEXP ff_, SEXP index_, SEXP nreturn_, SEXP value_);
 SEXP r_ff__get_contiguous(      SEXP ffmode_, SEXP ff_, SEXP index_, SEXP nreturn_             );
 SEXP r_ff__set_contiguous(      SEXP ffmode_, SEXP ff_, SEXP index_, SEXP nreturn_, SEXP value_);
@@ -211,8 +217,114 @@ SEXP r_ff_raw_addgetset_contiguous(SEXP ff_, SEXP index_, SEXP nreturn_, SEXP va
 SEXP r_ff_raw_addset_contiguous(   SEXP ff_, SEXP index_, SEXP nreturn_, SEXP value_);
 #endif
 
+/* } -- getset_contiguous ---------------------------------------------------------- */
 
 
+/* { -- getset_vec ---------------------------------------------------------- */
+
+SEXP r_ff__getset_vec(   SEXP ffmode_, SEXP ff_, SEXP index_, SEXP nreturn_, SEXP value_);
+SEXP r_ff__get_vec(      SEXP ffmode_, SEXP ff_, SEXP index_, SEXP nreturn_             );
+SEXP r_ff__set_vec(      SEXP ffmode_, SEXP ff_, SEXP index_, SEXP nreturn_, SEXP value_);
+SEXP r_ff__addgetset_vec(SEXP ffmode_, SEXP ff_, SEXP index_, SEXP nreturn_, SEXP value_);
+SEXP r_ff__addset_vec(   SEXP ffmode_, SEXP ff_, SEXP index_, SEXP nreturn_, SEXP value_);
+
+#if VMODE_COMPILE_BOOLEAN
+SEXP r_ff_boolean_getset_vec(   SEXP ff_, SEXP index_, SEXP nreturn_, SEXP value_);
+SEXP r_ff_boolean_get_vec(      SEXP ff_, SEXP index_, SEXP nreturn_             );
+SEXP r_ff_boolean_set_vec(      SEXP ff_, SEXP index_, SEXP nreturn_, SEXP value_);
+SEXP r_ff_boolean_addgetset_vec(SEXP ff_, SEXP index_, SEXP nreturn_, SEXP value_);
+SEXP r_ff_boolean_addset_vec(   SEXP ff_, SEXP index_, SEXP nreturn_, SEXP value_);
+#endif
+#if VMODE_COMPILE_LOGICAL
+SEXP r_ff_logical_getset_vec(   SEXP ff_, SEXP index_, SEXP nreturn_, SEXP value_);
+SEXP r_ff_logical_get_vec(      SEXP ff_, SEXP index_, SEXP nreturn_             );
+SEXP r_ff_logical_set_vec(      SEXP ff_, SEXP index_, SEXP nreturn_, SEXP value_);
+SEXP r_ff_logical_addgetset_vec(SEXP ff_, SEXP index_, SEXP nreturn_, SEXP value_);
+SEXP r_ff_logical_addset_vec(   SEXP ff_, SEXP index_, SEXP nreturn_, SEXP value_);
+#endif
+#if VMODE_COMPILE_QUAD
+SEXP r_ff_quad_getset_vec(   SEXP ff_, SEXP index_, SEXP nreturn_, SEXP value_);
+SEXP r_ff_quad_get_vec(      SEXP ff_, SEXP index_, SEXP nreturn_             );
+SEXP r_ff_quad_set_vec(      SEXP ff_, SEXP index_, SEXP nreturn_, SEXP value_);
+SEXP r_ff_quad_addgetset_vec(SEXP ff_, SEXP index_, SEXP nreturn_, SEXP value_);
+SEXP r_ff_quad_addset_vec(   SEXP ff_, SEXP index_, SEXP nreturn_, SEXP value_);
+#endif
+#if VMODE_COMPILE_NIBBLE
+SEXP r_ff_nibble_getset_vec(   SEXP ff_, SEXP index_, SEXP nreturn_, SEXP value_);
+SEXP r_ff_nibble_get_vec(      SEXP ff_, SEXP index_, SEXP nreturn_             );
+SEXP r_ff_nibble_set_vec(      SEXP ff_, SEXP index_, SEXP nreturn_, SEXP value_);
+SEXP r_ff_nibble_addgetset_vec(SEXP ff_, SEXP index_, SEXP nreturn_, SEXP value_);
+SEXP r_ff_nibble_addset_vec(   SEXP ff_, SEXP index_, SEXP nreturn_, SEXP value_);
+#endif
+#if VMODE_COMPILE_BYTE
+SEXP r_ff_byte_getset_vec(   SEXP ff_, SEXP index_, SEXP nreturn_, SEXP value_);
+SEXP r_ff_byte_get_vec(      SEXP ff_, SEXP index_, SEXP nreturn_             );
+SEXP r_ff_byte_set_vec(      SEXP ff_, SEXP index_, SEXP nreturn_, SEXP value_);
+SEXP r_ff_byte_addgetset_vec(SEXP ff_, SEXP index_, SEXP nreturn_, SEXP value_);
+SEXP r_ff_byte_addset_vec(   SEXP ff_, SEXP index_, SEXP nreturn_, SEXP value_);
+#endif
+#if VMODE_COMPILE_UBYTE
+SEXP r_ff_ubyte_getset_vec(   SEXP ff_, SEXP index_, SEXP nreturn_, SEXP value_);
+SEXP r_ff_ubyte_get_vec(      SEXP ff_, SEXP index_, SEXP nreturn_             );
+SEXP r_ff_ubyte_set_vec(      SEXP ff_, SEXP index_, SEXP nreturn_, SEXP value_);
+SEXP r_ff_ubyte_addgetset_vec(SEXP ff_, SEXP index_, SEXP nreturn_, SEXP value_);
+SEXP r_ff_ubyte_addset_vec(   SEXP ff_, SEXP index_, SEXP nreturn_, SEXP value_);
+#endif
+#if VMODE_COMPILE_SHORT
+SEXP r_ff_short_getset_vec(   SEXP ff_, SEXP index_, SEXP nreturn_, SEXP value_);
+SEXP r_ff_short_get_vec(      SEXP ff_, SEXP index_, SEXP nreturn_             );
+SEXP r_ff_short_set_vec(      SEXP ff_, SEXP index_, SEXP nreturn_, SEXP value_);
+SEXP r_ff_short_addgetset_vec(SEXP ff_, SEXP index_, SEXP nreturn_, SEXP value_);
+SEXP r_ff_short_addset_vec(   SEXP ff_, SEXP index_, SEXP nreturn_, SEXP value_);
+#endif
+#if VMODE_COMPILE_USHORT
+SEXP r_ff_ushort_getset_vec(   SEXP ff_, SEXP index_, SEXP nreturn_, SEXP value_);
+SEXP r_ff_ushort_get_vec(      SEXP ff_, SEXP index_, SEXP nreturn_             );
+SEXP r_ff_ushort_set_vec(      SEXP ff_, SEXP index_, SEXP nreturn_, SEXP value_);
+SEXP r_ff_ushort_addgetset_vec(SEXP ff_, SEXP index_, SEXP nreturn_, SEXP value_);
+SEXP r_ff_ushort_addset_vec(   SEXP ff_, SEXP index_, SEXP nreturn_, SEXP value_);
+#endif
+#if VMODE_COMPILE_INTEGER
+SEXP r_ff_integer_getset_vec(   SEXP ff_, SEXP index_, SEXP nreturn_, SEXP value_);
+SEXP r_ff_integer_get_vec(      SEXP ff_, SEXP index_, SEXP nreturn_             );
+SEXP r_ff_integer_set_vec(      SEXP ff_, SEXP index_, SEXP nreturn_, SEXP value_);
+SEXP r_ff_integer_addgetset_vec(SEXP ff_, SEXP index_, SEXP nreturn_, SEXP value_);
+SEXP r_ff_integer_addset_vec(   SEXP ff_, SEXP index_, SEXP nreturn_, SEXP value_);
+#endif
+#if VMODE_COMPILE_SINGLE
+SEXP r_ff_single_getset_vec(   SEXP ff_, SEXP index_, SEXP nreturn_, SEXP value_);
+SEXP r_ff_single_get_vec(      SEXP ff_, SEXP index_, SEXP nreturn_             );
+SEXP r_ff_single_set_vec(      SEXP ff_, SEXP index_, SEXP nreturn_, SEXP value_);
+SEXP r_ff_single_addgetset_vec(SEXP ff_, SEXP index_, SEXP nreturn_, SEXP value_);
+SEXP r_ff_single_addset_vec(   SEXP ff_, SEXP index_, SEXP nreturn_, SEXP value_);
+#endif
+#if VMODE_COMPILE_DOUBLE
+SEXP r_ff_double_getset_vec(   SEXP ff_, SEXP index_, SEXP nreturn_, SEXP value_);
+SEXP r_ff_double_get_vec(      SEXP ff_, SEXP index_, SEXP nreturn_             );
+SEXP r_ff_double_set_vec(      SEXP ff_, SEXP index_, SEXP nreturn_, SEXP value_);
+SEXP r_ff_double_addgetset_vec(SEXP ff_, SEXP index_, SEXP nreturn_, SEXP value_);
+SEXP r_ff_double_addset_vec(   SEXP ff_, SEXP index_, SEXP nreturn_, SEXP value_);
+#endif
+#if VMODE_COMPILE_COMPLEX
+SEXP r_ff_complex_getset_vec(   SEXP ff_, SEXP index_, SEXP nreturn_, SEXP value_);
+SEXP r_ff_complex_get_vec(      SEXP ff_, SEXP index_, SEXP nreturn_             );
+SEXP r_ff_complex_set_vec(      SEXP ff_, SEXP index_, SEXP nreturn_, SEXP value_);
+SEXP r_ff_complex_addgetset_vec(SEXP ff_, SEXP index_, SEXP nreturn_, SEXP value_);
+SEXP r_ff_complex_addset_vec(   SEXP ff_, SEXP index_, SEXP nreturn_, SEXP value_);
+#endif
+#if VMODE_COMPILE_RAW
+SEXP r_ff_raw_getset_vec(   SEXP ff_, SEXP index_, SEXP nreturn_, SEXP value_);
+SEXP r_ff_raw_get_vec(      SEXP ff_, SEXP index_, SEXP nreturn_             );
+SEXP r_ff_raw_set_vec(      SEXP ff_, SEXP index_, SEXP nreturn_, SEXP value_);
+SEXP r_ff_raw_addgetset_vec(SEXP ff_, SEXP index_, SEXP nreturn_, SEXP value_);
+SEXP r_ff_raw_addset_vec(   SEXP ff_, SEXP index_, SEXP nreturn_, SEXP value_);
+#endif
+
+/* } -- getset_vec ---------------------------------------------------------- */
+
+
+/* { -- getset_vector ---------------------------------------------------------- */
+
 //SEXP r_ff_double_getset_vector_iteration(SEXP ff_, SEXP index_, SEXP nreturn_, SEXP value_); /* dito, not used: demonstrating basic iterators */
 SEXP r_ff__getset_vector(   SEXP ffmode_, SEXP ff_, SEXP index_, SEXP nreturn_, SEXP value_);
 SEXP r_ff__get_vector(      SEXP ffmode_, SEXP ff_, SEXP index_, SEXP nreturn_             );
@@ -312,8 +424,10 @@ SEXP r_ff_raw_addgetset_vector(SEXP ff_, SEXP index_, SEXP nreturn_, SEXP value_
 SEXP r_ff_raw_addset_vector(   SEXP ff_, SEXP index_, SEXP nreturn_, SEXP value_);
 #endif
 
+/* } -- getset_vector ---------------------------------------------------------- */
 
 
+/* { -- getset_array ---------------------------------------------------------- */
 
 SEXP r_ff__getset_array(    SEXP ffmode_, SEXP ff_, SEXP index_, SEXP indexdim_, SEXP ffdim_, SEXP ndim_, SEXP nreturn_, SEXP cumindexdim_, SEXP cumffdim_, SEXP value_);
 SEXP r_ff__get_array(       SEXP ffmode_, SEXP ff_, SEXP index_, SEXP indexdim_, SEXP ffdim_, SEXP ndim_, SEXP nreturn_, SEXP cumindexdim_, SEXP cumffdim_             );
@@ -413,3 +527,4 @@ SEXP r_ff_raw_addgetset_array( SEXP ff_, SEXP index_, SEXP indexdim_, SEXP ffdim
 SEXP r_ff_raw_addset_array(    SEXP ff_, SEXP index_, SEXP indexdim_, SEXP ffdim_, SEXP ndim_, SEXP nreturn_, SEXP cumindexdim_, SEXP cumffdim_, SEXP value_);
 #endif
 
+/* } -- getset_array ---------------------------------------------------------- */
